
# THIS FILE IS GENERATED FROM ORANGE3-TEXT SETUP.PY
short_version = '0.6.0'
version = '0.6.0'
full_version = '0.6.0'
git_revision = '5ada43415c5430971ddee70f698ab714ba8619ea'
release = True
if not release:
    version = full_version
    short_version += ".dev"
